/* ptsager.c -- The Problem Tracking System problem-ager/auto-mailer.
 * Authors: Brian Goff & Dean Collins
 * Last Modified: Sat Mar 27 15:20:27 PST 1993
 */

/* Copyright 1993 Brian Goff, Dean Collins.
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notices appear in all copies and
 * that both those copyright notices and this permission notice appear in
 * supporting documentation, and that neither the name of the University of
 * Idaho nor the name of Dean Collins nor the name of Brian Goff
 * be used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission from both parties.
 * Neither The University of Idaho nor Dean Collins nor Brian Goff make
 * any representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE UNIVERSITY OF IDAHO, DEAN COLLINS AND BRIAN GOFF DISCLAIM ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL THE
 * UNIVERSITY OF IDAHO OR DEAN COLLINS BE LIABLE FOR ANY SPECIAL, INDIRECT
 * OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>              /* Standard input/output utilities hdr. file */
#include <sys/param.h>          /* System parameters header file */
#include <types.h>		/* Standard types header file */
#include <sys/time.h>		/* System time header file */
#include <time.h>               /* time.h - ANSI C, DATE and TIME header file.*/
#include <limits.h>		/* Sizes, limits, etc. */
#include "zdbm.h"               /* Zombie Database Manager header file */
#include "cloud.h"              /* Nebulous Cloud header file */
#include "clouderror.h"         /* Nebulous Cloud error rtn. header file */
#include "config.h"             /* Config. file parser header file */

/* This undef is important.  util.c will not compile correctly for
 * ptsager if XPTS is defined.
 */
#undef XPTS
#include "util.h"


#define MAIL_HEADER \
"\nNOTICE: These PTS problems are unsolved and are %d days old:\n\n"

#define MAIL_ADM_COMMAND \
"/usr/ucb/Mail -s \"PTS Unsolved Problems Notification\" %s < %s"

#define DAYLEN  3
#define YEARLEN 4

/* This may be necessary on SunOS and HP-UX */
#ifndef ARG_MAX
#ifdef _POSIX_ARG_MAX
#define ARG_MAX _POSIX_ARG_MAX
#endif
#endif

/* This may be necessary on SunOS and HP-UX */
#ifndef FILENAME_MAX
#ifdef _POSIX_PATH_MAX
#define FILENAME_MAX  _POSIX_PATH_MAX
#endif
#endif



/*
 * MailSysAdm[i].TmpFName is the temporary mail file for person i.
 * MailSysAdm[i].Flag says whether temporary mail file for person i
 * has been written to (other than header).
 */
typedef struct
{       int Flag ;
        char TmpFName[FILENAME_MAX] ;
} MailSysAdmRec ;


/* FUNCTION PROTOTYPES */
int main(int argc, char *argv[]) ;
void WriteHeader(char *fname, char *header, int days) ;
int CountRecs(MailInfo *list) ;
void PrintSummaries(Summary *CloudUnSummaries, char **UnsolvedList) ;
void GrabSysDate(int *yr, int *day) ;
void ParseDate(char *prid, int *yr, int *day) ;
void AppendProb(char *fname, char *path, char *summary) ;
void MailAdm(char *user, char *fname) ;


/*
______________________________________________________________________
main()

FUNCTION DESCRIPTION:

	This is the main function for the PTS problem-ager/
auto-mailer.  It finds problems that are X days old and sends
mail about them to user Y.  X & Y come from the PTS config file.
______________________________________________________________________
REVISION HISTORY:
     Authors:  Dean Collins & Brian Goff      Date:  2/28/93
     Documentation Enhancement: DC		     3/11/93
     Documentation Enhancement: DC		     3/27/93
______________________________________________________________________
*/

int
main(int argc,
     char *argv[])

     /* Interface description:
         INPUT:
          argc  - The number of command-line arguments. (UNUSED)
          argv  - An array of command-line arguments. (UNUSED)
         OUTPUT:
          None.
         RETURN VALUE:
          int   - Status flag.  Non-zero is error.
     */
      
{
     /* Internal variables: */
          /* Major */
   char **UnsolvedList = NULL ;		/* The list of unsolved problems */
   Summary *CloudUnSummaries = NULL ;	/* A ptr. to the unsolved problems
					 * linked list (used by neb_cld).
					 */
   Summary *s = NULL ;			/* The list of problem summaries */
   ProblemTree *ProbTree ;		/* The problem types tree */
   UserInfo UserInfoRec ;		/* Information about the user */
   problem_record *ProblemRecord ;	/* A full problem record */
   MailInfo *MailList=NULL ;		/* A list of people to send mail to. */
   MailSysAdmRec *MailSysAdm=NULL ;	/* A list of mail filenames */
   int NumAdmRecs=0 ;			/* Counter of # of MailSysAdm records*/
   int CurYear, CurDay ;		/* current year and day of year */
   int ProbYear, ProbDay ;		/* year and day of year a problem
					 * was reported
					 */

          /* Minor */
   MailInfo *p=NULL ;			/* A temporary mail list pointer */
   int age ;				/* The age of a problem */
   int i ;				/* Misc counter. */

   /*---------------------- Start of main() routine -----------------------*/

     /* Get information about the user and read the problem hierarchy. */
#ifdef DEBUG
   printf("calling Setup()...\n") ;
   printf("loading config file...\n") ;
#endif
/*CLD*/   if (Setup(&UserInfoRec, &ProbTree) == False)
   {  fprintf(stderr, "Error in configuration file %s !\n", CONFIGFILE) ;
      exit(1);
   }
#ifdef DEBUG
   printf("returned from Setup()...\n") ;
   printf("\tUserInfoRec.sysopflag = %d\n", UserInfoRec.sysopflag);
   printf("\tUserInfoRec.username  = %s\n", UserInfoRec.username) ;
   printf("\tUserInfoRec.realname  = %s\n", UserInfoRec.realname) ;
#endif

     /* Read the mail list from the config file. */
#ifdef DEBUG
   printf("Calling config_mail()\n") ;
#endif
   config_mail(&MailList) ;
#ifdef DEBUG
   printf("Returned from config_mail(), MailList=%p\n",MailList) ;
   p=MailList ;
   while (p)
   {  printf("name=%s\tdays=%d\n",p->username,p->days) ;
      p=p->next ;
   }
#endif


     /* Read all unsolved problems */
#ifdef DEBUG
   printf("searching for all unsolved problems...\n") ;
#endif
   CloudUnSummaries = ReadUnsolvedSummaries(ProbTree, &UnsolvedList) ;

#ifdef DEBUG
   printf("Checking for NULL values\n") ;
   if (ProbTree == NULL) printf("ProbTree is NULL\n") ;
   if (CloudUnSummaries== NULL) printf("CloudUnSummaries is NULL\n") ;
   if (UnsolvedList == NULL) printf("UnsolvedList is NULL\n") ;
   else if (UnsolvedList[0] == NULL) printf("UnsolvedList[0] is NULL\n") ;
#endif

   if (!CloudUnSummaries)
   {  printf("No unsolved problems found.\n") ;
      exit (0);
   }

     /* Count number of people in MailList and allocate an array of
      * MailSysAdm records.  These contain TmpFName and Flag.
      * MailSysAdm[i].TmpFName is the temporary mail file for person i.
      * MailSysAdm[i].Flag says whether temporary mail file for person i
      * has been written to (other than header).
      */
   NumAdmRecs = CountRecs(MailList) ;

     /* Allocate memory for array of MailSysAdm records.  This should have 
      * enough room for NumAdmRecs records.
      */
   MailSysAdm = calloc(NumAdmRecs+1, sizeof(MailSysAdmRec)) ;

     /* Fill in MailSysAdm records, setting Flag fields to 0 and
      * fill in the TmpFName fields with names created by tmpnam() .
      * Write temp file headers for each filename in MailSysAdm array.
      */
   for (i=0, p=MailList ; (i<NumAdmRecs) && (p != NULL); ++i, p=p->next)
   {   MailSysAdm[i].Flag = 0 ;
       strcpy(MailSysAdm[i].TmpFName, tmpnam(NULL)) ;

       /* Write temp file headers for each filename in MailSysAdm array */
#ifdef DEBUG
printf("writing header for (%s,%d)\n",MailSysAdm[i].TmpFName,p->days) ;
#endif
       WriteHeader(MailSysAdm[i].TmpFName, MAIL_HEADER, p->days) ;
   }


   /* Grab System Date */
   GrabSysDate(&CurYear, &CurDay) ;


      /* For each unsolved problem
       * 	Grab date for the problem (parse out fields)
       *	Compare date to current date (get back number of DaysOld)
       *	
       *	For each mail person
       *		Compare DaysOld to their day value
       *		If same,
       *		    Append that problem to their MailSysAdm mail file
       *		    Set their MailSysAdm flag to TRUE
       *		Endif
       *	Done
       * Done
       */
   for (s=CloudUnSummaries; s != NULL; s=s->next)
   {
      ParseDate(s->prid,&ProbYear,&ProbDay) ;

      age = DaysOld(ProbYear,ProbDay,CurYear,CurDay) ;

#ifdef DEBUG
      printf("age is %d\n",age) ;
#endif


      for (i=0, p=MailList ; (i<NumAdmRecs) && (p != NULL); ++i, p=p->next)
      {   if (age == p->days)
          {  AppendProb(MailSysAdm[i].TmpFName, s->path, s->summary) ;
             MailSysAdm[i].Flag = TRUE ;
          }
      }
   }



   /*
    * For each mail person
    *	If their MailSysAdm flag is TRUE
    *	    mail that person's MailSysAdm file to them
    *	    remove the temporary mail file
    *	Endif
    * Done
    */
   for (i=0, p=MailList; (i<NumAdmRecs) && (p != NULL); ++i, p=p->next)
   {
       if (MailSysAdm[i].Flag)
          MailAdm(p->username, MailSysAdm[i].TmpFName) ;

       unlink(MailSysAdm[i].TmpFName) ;

#ifdef DEBUG
       printf("Unlinked %s\n",MailSysAdm[i].TmpFName) ;
#endif
   }


}  /*-------------------------- End of main() -----------------------------*/

/*
______________________________________________________________________
MailAdm()

FUNCTION DESCRIPTION:

	Mail a file to to a system user.
______________________________________________________________________
REVISION HISTORY:
     Authors:  Dean Collins & Brian Goff      Date:  2/28/93
     Documentation Enhancement: DC		     3/27/93
______________________________________________________________________
*/


void
MailAdm(char *user, char *fname)

     /* Interface description:
         INPUT:
          user   - account name of user to mail to
          fname  - file to mail to user
         OUTPUT:
          None.
         RETURN VALUE:
          None.
     */
      
{
     /* Internal variables: */
          /* Major */
   char cmd[ARG_MAX] ; /*BIG!!*/

          /* Minor */


   /*------------------- Start of MailAdm() routine --------------------*/

     /* Put the command to mail the file in string "cmd" */
   sprintf(cmd, MAIL_ADM_COMMAND,
           user, fname) ;

#ifdef DEBUG
   printf("MailAdm(): cmd='%s'\n",cmd) ;
#endif

     /* Use system() to execute the command */
   system(cmd) ;

#ifdef DEBUG
/*   printf("Sent mail to %s\n",p->username) ;*/
#endif

}  /*----------------------- End of MailAdm() --------------------------*/

/*
______________________________________________________________________
CountRecs()

FUNCTION DESCRIPTION:
	Count the number of records in a MailInfo linked list.
If list is NULL it will return zero.
______________________________________________________________________
REVISION HISTORY:
     Authors:  Dean Collins & Brian Goff      Date:  2/28/93
     Documentation Enhancement: DC		     3/27/93
______________________________________________________________________
*/



int CountRecs(MailInfo *list)

     /* Interface description:
         INPUT:
          list  -  the MailInfo list to count
         OUTPUT:
          None.
         RETURN VALUE:
          int   -  then number of records in the list
     */
      
{
     /* Internal variables: */
          /* Major */
          /* NONE */

          /* Minor */
   int count=0 ;
   MailInfo *p ;


   /*------------------- Start of CountRecs() routine --------------------*/

     /* For each record in the MailInfo list, increment count */
   p=list ;
   while (p)
   {  ++count ;
      p=p->next ;
   }

#ifdef DEBUG
   printf("CountRecs returning: %d\n",count) ;
#endif


   return(count) ;

}  /*----------------------- End of CountRecs() --------------------------*/

/*
______________________________________________________________________
WriteHeader()

FUNCTION DESCRIPTION:
	Write the header for a mail file.
______________________________________________________________________
REVISION HISTORY:
     Authors:  Dean Collins & Brian Goff      Date:  2/28/93
     Documentation Enhancement: DC		     3/27/93
______________________________________________________________________
*/



void
WriteHeader(char *fname, char *header, int days)

     /* Interface description:
         INPUT:
          fname   - filename of mail file
          header  - header to write to fname (should contain one "%s" for days)
          days    - "days old" value to print in header
         OUTPUT:
          None.
         RETURN VALUE:
          None.
     */
      
{
     /* Internal variables: */
          /* Major */
   FILE *fp ;

          /* Minor */


   /*------------------- Start of WriteHeader() routine --------------------*/

#ifdef DEBUG
   printf("Writing header '%s' to file '%s'\n",header, fname) ;
#endif

     /* Open the file fname for writing */
   if ((fp=fopen(fname, "w"))== NULL)
   {  fprintf(stderr,"Error opening file %s\n",fname) ;
      exit(1) ;
   }


     /* Write the header */
   fprintf(fp, header, days) ;

     /* Close the file */
   fclose(fp) ;

}  /*----------------------- End of WriteHeader() --------------------------*/



/*
______________________________________________________________________
GrabSysDate()

FUNCTION DESCRIPTION:

	Obtain the system date.
______________________________________________________________________
REVISION HISTORY:
     Authors:  Dean Collins & Brian Goff      Date:  2/28/93
     Documentation Enhancement: DC		     3/27/93
______________________________________________________________________
*/



void GrabSysDate(int *yr, int *day)

     /* Interface description:
         INPUT:
          yr    - ptr to current system year
          day   - ptr to current system day of year (starting at 1)
         OUTPUT:
          None.
         RETURN VALUE:
          None.
     */
      
{
     /* Internal variables: */
          /* Major */
   struct timeval curdate;        /* current system date */
   struct timezone tz;		  /* current time zone */
   struct tm *localdate ;

          /* Minor */


   /*------------------ Start of GrabSysDate() routine -------------------*/

   gettimeofday(&curdate, &tz) ;

   localdate=localtime(&(curdate.tv_sec)) ;

   *yr = 1900 + localdate->tm_year ;

	/* tm_yday is 0 to 365.  We need to compare with strftime's 
	 * %j, which is 1 to 366.  ho hum.
	 */
   *day = localdate->tm_yday + 1 ; 

#ifdef DEBUG
   printf("Local Time (day/yr): %d/%d\n",*day,*yr) ;
#endif
}  /*----------------------- End of GrabSysDate() -----------------------*/

/*
______________________________________________________________________
ParseDate()

FUNCTION DESCRIPTION:

	Obtain the date associated with a problem.
______________________________________________________________________
REVISION HISTORY:
     Authors:  Dean Collins & Brian Goff      Date:  2/28/93
     Documentation Enhancement: DC		     3/27/93
______________________________________________________________________
*/



void ParseDate(char *prid, int *yr, int *day)

     /* Interface description:
         INPUT:
          prid  - ptr. to a problem ID
         OUTPUT:
          yr    - ptr. to the year the problem was reported
          day   - ptr. to the day of the year the problem was reported
			(starting at 1)
         RETURN VALUE:
          None.
     */
      
{
     /* Internal variables: */
          /* Major */
   char daystr[MAXPRID] ;
   char yrstr[MAXPRID] ;
   char *p ;

          /* Minor */


   /*------------------- Start of ParseDate() routine --------------------*/

     /* Copy the day of the year from the problem ID into daystr */
   strncpy(daystr,prid,DAYLEN) ;
   daystr[DAYLEN] = '\0' ;

     /* Copy the the year from the problem ID into yrstr */
   p = &(prid[DAYLEN]) ;
   strncpy(yrstr,p,YEARLEN) ;
   yrstr[YEARLEN] = '\0' ;

     /* Convert to integers and store in yr & day */
   *yr = atoi(yrstr) ;
   *day = atoi(daystr) ;


#ifdef DEBUG
   printf("ParseDate: daystr='%s', yrstr='%s, day=%d, yr=%d', prid=%s\n",
          daystr,yrstr,*day,*yr,prid) ;
#endif

}  /*----------------------- End of ParseDate() --------------------------*/



/*
______________________________________________________________________
DaysOld()

FUNCTION DESCRIPTION:

	Determines the difference (in days) between two dates.
Differences of more than 365 days are ignored.
______________________________________________________________________
REVISION HISTORY:
     Authors:  Dean Collins & Brian Goff      Date:  2/28/93
     Documentation Enhancement: DC		     3/27/93
______________________________________________________________________
*/

int DaysOld(int ProbYear, int ProbDay, int CurYear, int CurDay)

     /* Interface description:
         INPUT:
          ProbYear - The year associated with a problem
          ProbDay  - The day of the year associated with a problem
          CurYear  - The current year
          CurDay   - The current day of the year
         OUTPUT:
          None.
         RETURN VALUE:
          int      - Difference (in days) between the two dates.
			zero is returned if the difference is greater
			then 365 days.
     */
      
{
     /* Internal variables: */
          /* Major */
   int delta ;		/* Difference (in days) between the two dates. */

          /* Minor */


   /*---------------------- Start of DaysOld() routine --------------------*/

     /* If the two years are the same,
      *		delta = CurDay - Probday ;
      * If the two days differ by one,
      *         delta = YearLength(ProbYear) - ProbDay + CurDay ;
      * Otherwise,
      * 	delta = 0 ;
      */

   switch(CurYear - ProbYear)
   {
      case 0:   delta = CurDay - ProbDay ;
                break ;

      case 1:   delta = YearLength(ProbYear) - ProbDay + CurDay ;
                break ;

      default:  delta = 0 ;
   }

   return(delta) ;

}  /*-------------------------- End of DaysOld() --------------------------*/


/*
______________________________________________________________________
YearLength()

FUNCTION DESCRIPTION:
	Computes the number of days in a given year, accounting
for leap years.
______________________________________________________________________
REVISION HISTORY:
     Authors:  Dean Collins & Brian Goff      Date:  2/28/93
     Documentation Enhancement: DC		     3/27/93
______________________________________________________________________
*/



int YearLength(int year)

     /* Interface description:
         INPUT:
          year  - The year in question.
         OUTPUT:
          None.
         RETURN VALUE:
          int   - The number if days in the year.
     */
      
{
     /* Internal variables: */
          /* Major */
   int length ;

          /* Minor */


   /*------------------- Start of YearLength() routine --------------------*/

     /* If the year is not evenly divisible by four,
      *		length = 365 ;
      * else if the year is not evenly divisible by 400 (is a leap year),
      *          length = 365 ;
      * else length = 366 ;
      */


#ifdef DEBUG
printf("year%%4=%d\n",year%4) ;
#endif
   if (year % 4) length = 365 ;

   else if (year % 400) length = 365 ;
        else length = 366 ;

     /* Yes, I know there's more to this... */

#ifdef DEBUG
   printf("YearLength(): Year = %d, Length = %d\n", year, length) ;
#endif

   return (length) ;

}  /*----------------------- End of YearLength() --------------------------*/

/*
______________________________________________________________________
AppendProb()

FUNCTION DESCRIPTION:

	Appends a problem summary (and it's problem type) to a file.
______________________________________________________________________
REVISION HISTORY:
     Authors:  Dean Collins & Brian Goff      Date:  2/28/93
     Documentation Enhancement: DC		     3/27/93
______________________________________________________________________
*/


void AppendProb(char *fname, char *path, char *summary)

     /* Interface description:
         INPUT:
          fname   - The name of the file to append to.
          path    - The problem tree path for the problem.
          summary - The summary for the problem.
         OUTPUT:
          None.
         RETURN VALUE:
          None.
     */
      
{
     /* Internal variables: */
          /* Major */
   FILE *fp ;

          /* Minor */


   /*------------------- Start of AppendProb() routine --------------------*/

     /* Open the file to write to. */
   if ((fp=fopen(fname, "a"))== NULL)
   {  fprintf(stderr,"Error opening file %s for append\n",fname) ;
      exit(1) ;
   }

     /* Print the summary and path to the file */
   fprintf(fp, "Summary:  %s\n", summary) ;
   fprintf(fp, "Path:     %s\n\n", path) ;

     /* Closwe the file */
   fclose(fp) ;

   
}  /*----------------------- End of AppendProb() --------------------------*/

/* End of ptsager.c */
