/*
 * edit_problem.c --
 * Chris Hyde
 * created: 
 */

/*
 * Copyright (c) 1992 Christopher G. Hyde
 * Copyright (c) 1992 University of Idaho, Moscow, Idaho.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation free of charge for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the University of Idaho
 * not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  The University
 * of Idaho makes no representations about the suitability of this software
 * for any purpose.  It is provided "as is" without express or implied warranty.
 *
 * THE UNIVERSITY OF IDAHO DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE UNIVERSITY OF IDAHO BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <errno.h>
#include <pwd.h>
#ifdef SUNOS
#include <unistd.h>
#endif /*SUNOS*/

#include "zdbm.h"

/* the only global variable in zdbm, it is set when an error occurs, it set
 * to either a standard errno (in the case of a an error of a library function
 * or to a zdbm error code (see zdbm.h) in the case of a zdbm error.
 *
 * this is defined as an extern in zdbm.h
 */

 int db_errorno;


int edit_log(char *db_path, char *prid, char *mod_log, int mode)
{
 char tprid[MAXPRID];
 char tpath[DBPATHLEN];
 index_record index;
 FILE *temp;

#ifdef DEBUG
 printf("\nIn edit_log()\n");
 printf("\t(Un)Marking problem %s%s/%s\n", PTSROOT, db_path, prid);
 printf("\tMarking mode = %d\n", mode);
#endif

 db_errorno=0;

 if ((temp=open_file(db_path, ".index", "r+", INDEX))==NULL)
    return(FAIL);

 if (find_problem(temp, prid, &index)==NULL)
 {	/* could not find problem requested */
    close_file(temp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("edit_log() - found problem to edit, or unedit\n");
#endif

 if (index.mrd==LINKED) /* problem does not reside in leaf given */
 {	/* so get the original problem record */

#ifdef DEBUG
 printf("edit_log() - problem given is a link\n");
#endif

/* since the call to find_problem() destroyes the data in index while it is 
 * trying to find the problem, the prid and the path of the original link 
 * need to be separate.
 */
    nl_strncpy(tprid, index.ln_prid, MAXPRID); 
    nl_strncpy(tpath, index.ln_path, DBPATHLEN);
    close_file(temp);

/* open the index of the original problem, and get it */
    if ((temp=open_file(tpath, ".index", "r+", INDEX))==NULL)
       return(FAIL);

    if (find_problem(temp, tprid, &index)==NULL)
    {	/* could not find problem requested */
       close_file(temp);
       return(FAIL);
    }

#ifdef DEBUG
 printf("edit_log() - found original problem\n");
#endif

 } /* end if LINKED problem */
 else
 {	/* we're using tpath and tprid for the edit log */
    nl_strncpy(tprid, prid, MAXPRID);
    nl_strncpy(tpath, db_path, DBPATHLEN);
 }	/* end else */
 
/* now that we have the real problem, check to be sure it can be edited */

#ifdef DEBUG
 printf("edit_log() - checking validity of call\n");
#endif

 if (index.mrd==GONE)
 {	/* problem was deleted */
    db_errorno=DELETED;
    close_file(temp);
    return(FAIL);
 }

 if (lock_file(temp)==FAIL)
 {	/* lock failed */
    close_file(temp);
    return(FAIL);
 }

/* problem can be edited, more or less, now perform action indicated by mode */
switch(mode)
{
    case START:		if (lock_for_edit(temp, &index)==FAIL)
			{	/* could not lock the record for editing */
			   unlock_file(temp);
			   close_file(temp);
			   return(FAIL);
			}
#ifdef DEBUG
 printf("edit_log() - started edit session\n");
#endif
			break;
   
    case END:		
			if (unlock_from_edit(temp, &index, tpath, mod_log)
				==FAIL)
			{	/* could not unlock the record */
			   unlock_file(temp);
			   close_file(temp);
			   return(FAIL);
			}
#ifdef DEBUG
 printf("edit_log() - ended edit session\n");
#endif
			break;

    default:		db_errorno=BADTYPE;
			unlock_file(temp);
			close_file(temp);
			return(FAIL);
 } /* end switch */
/* there were no errors, so clean up and return sucess */
 unlock_file(temp);
 close_file(temp);
 return(SUCESS);
} /* end edit_log */


int lock_for_edit(FILE *db_pointer, index_record *index)
{
 struct passwd *pwd;

#ifdef DEBUG
 printf("\nIn lock_for_edit()\n");
#endif

 if (index->ef)
 {	/* the problem is currently being edited */
    db_errorno=errno=ETXTBSY;
    return(FAIL);
 }

 if (index->sf==SOLVED)
 {	/* can't edit a solved problem */
    db_errorno=ISSOLVED;
    return(FAIL);
 }

/* OK to edit problem, so reset to the begining of the index entry and update
 * it.
 */

 fseek(db_pointer, -(long)(sizeof(index_record)), SEEK_CUR);

#ifdef DEBUG
 printf("lock_for_edit() - marking record as being in edit\n");
#endif

 index->pid=getpid();
 gethostname(index->node_name, MAXHOSTNAMELEN);
 pwd=getpwuid(geteuid());
 nl_strncpy(index->account_name, pwd->pw_name, MAXNAMELEN);
 index->ef=TRUE;

 if (fwrite(index, sizeof(index_record), 1, db_pointer)<1)
 {	/* error writting new entry */
    db_errorno=errno;
    return(FAIL);
 }
 else
 {

#ifdef DEBUG
 printf("lock_for_edit() - updated record as being in edit\n");
#endif

    return(SUCESS);
 } /* end else */
} /* end lock_for_edit */


int unlock_from_edit(FILE *db_pointer, index_record *index, char *db_path,
			char *new_log)
{
 struct passwd *pwd;

#ifdef DEBUG
 printf("\nIn unlock_from_edit()\n");
#endif

 if (index->ef==FALSE) /* file is not currently being edited */
 {
#ifdef DEBUG
 printf("unlock_from_edit() - problem is not being edited\n");
#endif

    return(SUCESS);
 }

 pwd=getpwuid(geteuid());
 if (strcmp(index->account_name, pwd->pw_name)!=0)
 {	/* only the person that started the edit can remove it */
    db_errorno=NOTOWNER;
    return(FAIL);
 }

 if (new_log!=NULL) /* there is something to write out */
 {
#ifdef DEBUG
 printf("unlock_from_edit() - writting out new problem log\n");
#endif

    if (write_log(db_path, index->prid, new_log)==FAIL)
       return(FAIL);

#ifdef DEBUG
 printf("unlock_from_edit() - wrote out new problem log\n");
#endif
 } /* end != NULL */
#ifdef DEBUG
 else
    printf("unlock_from_edit() - log is NULL\n");
#endif
 
/* we can remove the edit lock now, so do it */
 fseek(db_pointer, -(long)(sizeof(index_record)), SEEK_CUR);

 index->ef=FALSE;
#ifdef DEBUG
 printf("unlock_from_edit() - removing edit lock\n");
#endif

 if (fwrite(index, sizeof(index_record), 1, db_pointer)==NULL)
 {	/* write failed */
    db_errorno=errno;
    return(FAIL);
 }
 else
 {
#ifdef DEBUG
 printf("unlock_from_edit() - removed edit lock\n");
#endif

    return(SUCESS);
 } /* end else */
} /* end unlock_from_edit */


int remove_edit(char *db_path, char *prid)
{
 index_record index;
 FILE *temp;
 char tprid[MAXPRID];

#ifdef DEBUG
 printf("\nIn remove_edit()\n");
 printf("\tProblem to remove the edit lock from - %s/%s.\n", db_path,
		 prid);
#endif

 db_errorno=0;

 if ((temp=open_file(db_path, ".index", "r+", INDEX))==NULL)
    return(FAIL);

 if (find_problem(temp, prid, &index)==NULL)
 {	/* could not find problem specified */
    close_file(temp);
    return(FAIL);
 }

#ifdef DEBUG
 printf("remove_edit() - found problem given\n");
#endif

 if (index.mrd==LINKED) /* problem does not reside in leaf given */
 {

#ifdef DEBUG
 printf("remove_edit() - problem given is linked, getting original problem\n");
#endif

/* since find_problem() overwrites the data in index while searching, the
 * prid needs to be seaperate
 */

    close_file(temp);
    if ((temp=open_file(index.ln_path, ".index", "r+", INDEX))==NULL)
       return(FAIL);
    nl_strncpy(tprid, index.ln_prid, MAXPRID);
    if (find_problem(temp, tprid, &index)==NULL)
    {	/* could not find problem specified */
	close_file(temp);
	return(FAIL);
    }

#ifdef DEBUG
 printf("remove_edit() - found original problem\n");
#endif

 } /* end LINKED problem */

/* index is now the correct record, and temp is the right index file */

#ifdef DEBUG
 printf("remove_edit() - updating index entry\n");
#endif

 index.ef=FALSE;
 fseek(temp, -(long)(sizeof(index_record)), SEEK_CUR);
 if (lock_file(temp)==FAIL)
 {	/* the lock failed */
    close_file(temp);
    return(FAIL);
 }
 if (fwrite(&index, sizeof(index_record), 1, temp)==NULL)
 {	/* the write failed */
    db_errorno=errno;
    unlock_file(temp);
    close_file(temp);
    return(FAIL);
 }
 unlock_file(temp);
 close_file(temp);

#ifdef DEBUG
 printf("remove_edit() - updated index entry\n");
#endif

 return(SUCESS);
} /* end remove_edit */


edit_record *edit_info(char *db_path, char *prid)
{
 index_record index;
 static edit_record er;
 FILE *temp;

#ifdef DEBUG
 printf("\nIn edit_info()\n");
 printf("\tProblem to get edit info on  - %s/%s\n", db_path, prid);
#endif

 if ((temp=open_file(db_path, ".index", "r+", INDEX))==NULL)
    return(NULL);

 if (find_problem(temp, prid, &index)==NULL)
 {	/* could not find problem specified */
    close_file(temp);
    return(NULL);
 }

#ifdef DEBUG
 printf("edit_info() - found problem given\n");
 printf("edit_info() - creating edit_record\n");
#endif

/* now that we have an index record, copy the appropriate parts into
 * the info record and return it
 */

 er.ef=index.ef; /* the edit flag */ 
 /* who owns the edit */
 nl_strncpy(er.account_name, index.account_name, MAXNAMELEN);
 /* where is it running */
 nl_strncpy(er.node_name, index.node_name, MAXHOSTNAMELEN);
 er.pid=index.pid; /* what owns the edit */
 close_file(temp);
 return(&er);
} /* end edit_info */

/* end of edit_problem.c */
